--------------------------------------------------------------------------------
--
--                COPYRIGHT (C) 2000-2001 - LSI LOGIC.
--    All rights reserved. Reproduction in whole or part is prohibited
--          without the written permission of the copyright owner.
--
--------------------------------------------------------------------------------

---------------------------------------------------------------------------
--
-----------------------------------------------------------------------------
LIBRARY IEEE ;
--

USE ieee.std_logic_1164.all   ;
USE ieee.std_logic_arith.all  ;

-----------------------------------------------------------------------------

-----------------------------------------------------------------------------

package package_g12 is

                ---- Declaration section ----

                 ---- *** Type *** ----

  -- array of bit 
type array_8bit  is array(integer range<>) of std_ulogic_vector( 7 downto 0) ;
type array_12bit is array(integer range<>) of std_ulogic_vector(11 downto 0) ;
type array_15bit is array(integer range<>) of std_ulogic_vector(14 downto 0) ;

                 ---- *** Functions *** ----
-- 1) WindowH 
--
-- Window type
--   _              _
-- _| |_  start  __| |__ stop
--     ______________
--  __|              |___ 
-- 
-- Stop has higher priority on Start
--
   function WindowH(signal Stop,Start,Holdv : std_ulogic)
   return std_ulogic ;

-- 2) WindowL 
--
-- Window type
--   _              _
-- _| |_  start  __| |__ stop
--  ___                ___
--     |______________| 
-- 
-- Stop has higher priority on Start
--
   function WindowL(signal Stop,Start,Holdv : std_ulogic)
   return std_ulogic ;
   
                 ---- *** Procedure *** ----
                 
-- 1) Sqrt_Cell
--
--    this procedure implements the following operation :
--
--            A(n-1:0)    B(n-1:0)
--                 |           |
--                 |__________ | _____
--              ___|___________|___   |
--             |                   |  |
--  Cout <-----|       A + B       |  |
--          |  |___________________|  |
--          |            |    ________|
--          |          __|___|__
--          |_________|   mux   |
--                    |_________|
--                         | Remd(n-1:0) 

     procedure Sqrt_Cell(signal A,B  : in  std_ulogic_vector ;
                         signal Cout : out std_ulogic        ;
                         signal Remd : out std_ulogic_vector  ) ;
                   
   
end package_g12 ;

-----------------------------------------------------------------------------

package body package_g12 is

        ---- Functions ----
-- 1) WindowH

   function WindowH(signal Stop,Start,Holdv : std_ulogic)
   return std_ulogic is
   variable return_value : std_ulogic ;
   begin
      return_value := not(Stop or
                           not(Start or Holdv) ) ;
      return return_value ;
   end WindowH ;


-- 2) WindowL
   function WindowL(signal Stop,Start,Holdv : std_ulogic)
   return std_ulogic is
   variable return_value : std_ulogic ;
   begin
      return_value := Stop or
                       (not(Start) and Holdv) ;
      return return_value ;
   end WindowL ;

        ---- Procedure --
-- 1) Sqrt_Cell
     procedure Sqrt_Cell(signal A,B  : in  std_ulogic_vector ;
                         signal Cout : out std_ulogic        ;
                         signal Remd : out std_ulogic_vector  ) is
                         
     constant left_index : integer := A'left ;
     variable q_value    : std_ulogic        ;
     variable temp       : unsigned((left_index + 1) downto 0) ;
     begin
       temp    := unsigned('0' & A) + unsigned(B) ;
       
       q_value := temp(left_index + 1) ;
       Cout    <= q_value              ;
       
       if(q_value = '1') then
          Remd <= std_ulogic_vector(temp(left_index downto 0)) ;
       else
          Remd <= A ;
       end if ;
       
     end Sqrt_Cell ;



end package_g12 ;

-----------------------------------------------------------------------------
